"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const zhin_1 = require("zhin");
require("@zhinjs/plugin-http-server");
const client_1 = require("./client");
const message_1 = require("./message");
const adapter = new zhin_1.Adapter('com-wechat');
adapter.schema({
    type: zhin_1.Schema.const('ws', '连接方式'),
    url: zhin_1.Schema.string('请输入服务端ws地址'),
    access_token: zhin_1.Schema.string('请输入access_token'),
    max_reconnect_count: zhin_1.Schema.number('请输入max_reconnect_count').default(10),
    reconnect_interval: zhin_1.Schema.number('请输入reconnect_interval').default(3000),
});
adapter.define('sendMsg', async (bot_id, target_id, target_type, message, source) => {
    const bot = adapter.pick(bot_id);
    let msg = await adapter.app.renderMessage(message, source);
    msg = message_1.Message.formatSegments(msg);
    switch (target_type) {
        case 'group':
            return bot.sendGroupMsg(target_id, msg);
        case 'private':
            return bot.sendPrivateMsg(target_id, msg);
        default:
            throw new Error(`com wechat 适配器暂不支持发送${target_type}类型的消息`);
    }
});
const startBots = (configs) => {
    if (!adapter.app?.server)
        throw new Error('“com-wechat miss require service “http”, maybe you need install “ @zhinjs/plugin-http-server ”');
    for (const config of configs) {
        const bot = new client_1.Client(adapter, config, adapter.app.router);
        Object.defineProperties(bot, {
            unique_id: {
                get() {
                    return config.unique_id;
                },
            },
            quote_self: {
                get() {
                    return adapter.botConfig(bot)?.quote_self;
                },
            },
            forward_length: {
                get() {
                    return adapter.botConfig(bot)?.forward_length;
                },
            },
            command_prefix: {
                get() {
                    return adapter.botConfig(bot)?.command_prefix;
                },
            },
        });
        bot.on('message', messageHandler.bind(global, bot));
        bot.start().then(() => {
            adapter.emit('bot-ready', bot);
        });
        adapter.bots.push(bot);
    }
};
const messageHandler = (bot, event) => {
    const message = zhin_1.Message.fromEvent(adapter, bot, event);
    message.raw_message = message_1.Message.formatToString(event.message);
    message.message_type = event.detail_type;
    message.from_id =
        event.detail_type === 'private'
            ? event.user_id + ''
            : event.detail_type === 'group'
                ? event.group_id + ''
                : event.guild_id + '';
    const master = bot.config?.master;
    const admins = bot.config.admins || [];
    message.sender = {
        user_id: event.user_id,
        user_name: event.nickname || '',
        permissions: [
            master && event.user_id === master && 'master',
            admins && admins.includes(event.user_id) && 'admins',
            ...(event.permissions || []),
        ].filter(Boolean),
    };
    adapter.app.emit('message', adapter, bot, message);
};
const stopBots = () => {
    for (const bot of adapter.bots) {
        bot.stop();
    }
};
adapter.on('start', startBots);
adapter.on('stop', stopBots);
exports.default = adapter;
